using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;

namespace LocalTimeReader
{
    public partial class MainForm : Form
    {
        private LocalTimeReaderPort localTimeReaderPort = null;
        private int runningThreads;

        public MainForm()
        {
            InitializeComponent();

            runButton.Enabled = false;
            disconnectButton.Enabled = false;

            string[] names = SerialPort.GetPortNames();

            foreach (string name in names)
            {
                portComboBox.Items.Add(name);
                if (name == "COM1")
                    portComboBox.SelectedItem = name;
            }

            RefreshTabPages();
        }

        public void RunStop()
        {
            if (runningThreads > 0)
                runningThreads--;

            if (runningThreads == 0)
            {
                runButton.Enabled = true;
                updateButton.Enabled = true;
            }
        }

        public void RefreshTabPages()
        {
            tabControl.Controls.Clear();

            for (byte iii=0; iii < (byte)updateNumericUpDown.Value; iii++)
                tabControl.Controls.Add(new ThreadPage(iii, this));
        }

        public void ReceivedAcknowledgeRequest()
        {
            if (this.InvokeRequired)
                BeginInvoke((MethodInvoker)(delegate { ReceivedAcknowledgeRequest(); }));
            else
            {
                runButton.Enabled = true;
                localTimeReaderPort.SendAcknowledge();
            }
        }

        public void ReceivedTime(byte index, long localTime, long remoteTime)
        {
            if (this.InvokeRequired)
                BeginInvoke((MethodInvoker)(delegate { ReceivedTime(index, localTime, remoteTime); }));
            else
            {
                ((ThreadPage)tabControl.Controls[index]).ReceivedTime(localTime, remoteTime);
            }
        }

        private void runButton_Click(object sender, EventArgs e)
        {
            foreach (ThreadPage page in tabControl.Controls)
                page.Run(localTimeReaderPort, (int)intervalNumericUpDown.Value, (int)sampleNumericUpDown.Value);

            runningThreads = tabControl.Controls.Count;

            if ((int)sampleNumericUpDown.Value > 1)
            {
                updateButton.Enabled = false;
                runButton.Enabled = false;
            }

        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            if (localTimeReaderPort != null)
                localTimeReaderPort.ClosePort();

            localTimeReaderPort = new LocalTimeReaderPort(this);

            if (localTimeReaderPort.OpenPort(portComboBox.SelectedItem.ToString()))
            {
                connectButton.Enabled = false;
                disconnectButton.Enabled = true;
            }
            else
                MessageBox.Show("Port " + portComboBox.SelectedItem.ToString() + " failed to open");
        }

        private void disconnectButton_Click(object sender, EventArgs e)
        {
            if (localTimeReaderPort != null)
                localTimeReaderPort.ClosePort();
            PortClosing();
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            if (localTimeReaderPort != null)
                localTimeReaderPort.ClosePort();
            Application.Exit();
        }

        public void PortClosing()
        {
            if (this.InvokeRequired)
                BeginInvoke((MethodInvoker)(delegate { PortClosing(); }));
            else
            {
                localTimeReaderPort = null;
                runButton.Enabled = false;
                connectButton.Enabled = true;
                disconnectButton.Enabled = false;
            }
        }

        private void updateButton_Click(object sender, EventArgs e)
        {
            RefreshTabPages();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (localTimeReaderPort != null)
                localTimeReaderPort.ClosePort();
            Application.Exit();
        }

    }
}