// RemoteTimeReaderGTSC.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "conio.h"
#include "windows.h"
#include "winbase.h"
#include "process.h"
#include "queue"

using namespace std;

enum
{
    NotConnected,
    ConnectionEstablished,
    ConnectionIdle,
    CheckConnected
}status;

typedef struct
{
	int len;
	char data[18];
}Message;

queue <Message> sendQueue;
CRITICAL_SECTION criticalSection;
HANDLE hSerialPort;
HANDLE hQueueEvent;

void StartSendMessage(Message message)
{
	EnterCriticalSection(&criticalSection);
	sendQueue.push(message);
	LeaveCriticalSection(&criticalSection);

	SetEvent(hQueueEvent);
}

void SendInfoMessage(char* data)
{
	Message message;

	message.len = 18;

	message.data[0] = 0x05;
	memcpy(&(message.data[1]), &data[1], 9);

	SYSTEMTIME st;

	union
	{
		FILETIME ft;
		char bytes[8];
	}nowTime;

	GetLocalTime(&st);
	SystemTimeToFileTime(&st, &(nowTime.ft));

	memcpy(&(message.data[10]), nowTime.bytes, 8);

	StartSendMessage(message);
}

void SendOneByteMessage(char dataByte)
{
	Message message;

	message.len = 1;
	message.data[0] = dataByte;

//	cprintf("Sent %x\n", dataByte);
	StartSendMessage(message);
}

bool AcknowledgementTimerElapsed()
{
/*
	if (status == NotConnected)
		cprintf("Ack Time NotConnected\n");
	else if (status == ConnectionEstablished)
		cprintf("Ack Time ConnectionEstablished\n");
	else if (status ==	ConnectionIdle)
		cprintf("Ack Time ConnectionIdle\n");
	else if (status == CheckConnected)
		cprintf("Ack Time CheckConnected\n");
	else
		cprintf("Ack Time Invalid\n");
*/		

    if (status == ConnectionEstablished)
        status = ConnectionIdle;
    else if (status == ConnectionIdle || status == NotConnected)
    {
        if (status == ConnectionIdle)
            status = CheckConnected;

        SendOneByteMessage(0x01);
		return true;
    }
    else if (status == CheckConnected)
    {
        cprintf("Communication lost - Timeout on acknowledge request\n");
        return false;
    }

    return true;
}

void SendMessageThread(void* dummy)
{
	OVERLAPPED osWrite = {0};

	osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	while(true)
	{
		while(!sendQueue.empty())
		{
			Message message;

			EnterCriticalSection(&criticalSection);
			memcpy(&message, &((Message&)sendQueue.front()), sizeof(Message));
			sendQueue.pop();
			LeaveCriticalSection(&criticalSection);

			DWORD bytesSent;

			if (WriteFile(hSerialPort, message.data, message.len, &bytesSent, &osWrite) == 0)
			{
				if (GetLastError() != ERROR_IO_PENDING)
				{
					cprintf("Failed to send message\n");
					return;
				}
				else
					GetOverlappedResult(hSerialPort, &osWrite, &bytesSent, TRUE);
			}

			if (bytesSent != message.len)
			{
				cprintf("Failed to send complete message\n");
				return;
			}
		}

		WaitForSingleObject(hQueueEvent, INFINITE);
		ResetEvent(hQueueEvent);
	}

}

int _tmain(int argc, _TCHAR* argv[])
{
	char receivedMessage[10];
	int messageIndex = 0;
	status = NotConnected;

	hSerialPort = CreateFile(_T("COM1"), 
            GENERIC_READ | GENERIC_WRITE, 
            0, 
            NULL, 
            OPEN_EXISTING, 
            FILE_FLAG_OVERLAPPED, 
            NULL);

	if (hSerialPort == INVALID_HANDLE_VALUE)
	{
		cprintf("Cannot get serial port handle\n");
		return 0;
	}

	DCB dcbConfig;

	if(GetCommState(hSerialPort, &dcbConfig))
	{
		dcbConfig.BaudRate = 9600;
		dcbConfig.ByteSize = 8;
		dcbConfig.Parity = NOPARITY;
		dcbConfig.StopBits = ONESTOPBIT;
		dcbConfig.fDsrSensitivity = FALSE;
		dcbConfig.fDtrControl = DTR_CONTROL_ENABLE;
		dcbConfig.fBinary = TRUE;
		dcbConfig.fRtsControl = RTS_CONTROL_ENABLE;
		dcbConfig.fOutxCtsFlow = TRUE;
	}
	else
	{
		cprintf("Cannot get comm state\n");
		return 0;
	}

	if(!SetCommState(hSerialPort, &dcbConfig))
	{
		cprintf("Cannot set comm state\n");
		return 0;
	}

	COMMTIMEOUTS commTimeout;

	if(GetCommTimeouts(hSerialPort, &commTimeout)) /* Configuring Read & Write Time Outs */
  	{
   		commTimeout.ReadTotalTimeoutConstant = 5000;
  		commTimeout.ReadTotalTimeoutMultiplier = 0;
  		commTimeout.WriteTotalTimeoutConstant = 10000;
  		commTimeout.WriteTotalTimeoutMultiplier = 0;
  	} 
  	else
	{
		cprintf("Get timeout fail\n");
		return 0;
	}
  
  	if(!SetCommTimeouts(hSerialPort, &commTimeout))
	{
		cprintf("Set timeout fail\n");
		return 0;
	}

	InitializeCriticalSection(&criticalSection);
	hQueueEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	_beginthread(SendMessageThread, 0, NULL);

	OVERLAPPED osRead = {0};
	osRead.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
   
	while(true)
	{
		char buffer;
		DWORD readSize;

		if (ReadFile(hSerialPort, &buffer, 1, &readSize, &osRead) == 0)
		{
			if (GetLastError()!= ERROR_IO_PENDING)
			{
				cprintf("Read Error\n");
				break;
			}
			else
				GetOverlappedResult(hSerialPort, &osRead, &readSize, TRUE);
		}

		if (readSize != 0)
		{
			if (messageIndex == 0)
			{
				if (buffer == 0x02)
				{
					status = ConnectionEstablished;
					SendOneByteMessage(0x03);
				}

				if (status == NotConnected)
					continue;

				if (buffer == 0x04)
				{
					status = ConnectionEstablished;
					receivedMessage[0] = buffer;
					messageIndex++;
				}
			}
			else
			{
				status = ConnectionEstablished;
				receivedMessage[messageIndex] = buffer;
				messageIndex++;
				if (messageIndex == 10)
				{
					messageIndex = 0;
					SendInfoMessage(receivedMessage);
				}
			}
		}
		else if (!AcknowledgementTimerElapsed())
			break;

	}

	return 0;
}
